package unitTest;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertNull;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DynamicTest;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestFactory;
import org.junit.jupiter.api.function.Executable;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;

import exceptions.*;
import project.Auction;
import project.Category;
import project.Item;
import project.User;

public class UserUnitTest {


	@BeforeEach
	public void reset() {

		Item.items = new ArrayList<Item>();
		User.users = new ArrayList<User>();
		Auction.auctions = new ArrayList<Auction>();
		Category.categories = new ArrayList<Category>();

	}
	
	@Test
	public void loginTest()
			throws UsernameEmailEmptyException, PasswordInvalidException, MailInvalidException, LoginInvalidException {

		String username = "mario", password = "password", email = "test@gmail.com";
		User u = new User();
		u.setUsername(username);
		u.setEmail(email);
		u.setPassword(password);

		assertTrue(User.login(username, email, password));
		assertTrue(User.login("", email, password));
		assertThrows(LoginInvalidException.class, () -> User.login(username, email, "wrong password"));
		assertThrows(LoginInvalidException.class, () -> User.login("wrongUsername", email, "wrong password"));
		assertThrows(LoginInvalidException.class, () -> User.login("", "wrongemail@gmail.com", "wrong password"));
		assertThrows(LoginInvalidException.class, () -> User.login("", email, "wrong password"));

	}

	@TestFactory
	public Collection<DynamicTest> loginAcceptanceTest() {

		ArrayList<String> username = new ArrayList<String>(Arrays.asList("", "", "mario", "mario", ""));
		ArrayList<String> email = new ArrayList<String>(
				Arrays.asList("", "an.gmail.it", "mario@gmail.com", "", "mario@gmail.com"));
		ArrayList<String> password = new ArrayList<String>(
				Arrays.asList("test", "test", "", "wrong password", "wrong password"));
		ArrayList<String> testNames = new ArrayList<String>(Arrays.asList("username and mail empty",
				"wrong mail format", "password empty", "login invalid with username", "login invalid with email"));
		ArrayList<Class<? extends Exception>> exceptions = new ArrayList<Class<? extends Exception>>(
				Arrays.asList(UsernameEmailEmptyException.class, MailInvalidException.class,
						PasswordInvalidException.class, LoginInvalidException.class, LoginInvalidException.class));

		Collection<DynamicTest> dynamicTests = new ArrayList<DynamicTest>();

		for (int i = 0; i < username.size(); i++) {

			String us = username.get(i);
			String em = email.get(i);
			String pw = password.get(i);

			final int j = i;

			Executable exec = () -> assertThrows(exceptions.get(j), () -> User.login(us, em, pw));
			DynamicTest dTest = DynamicTest.dynamicTest(testNames.get(i), exec);

			dynamicTests.add(dTest);

		}

		return dynamicTests;

	}

	@Test
	public void searchItemTest() throws CategoryInvalidException, TitleInvalidException {

		String title = "Red bike", category = "Sport and Outdoor", category2 = "Cars and Motorbikes";
		User u = User.getInstance();
		new Category(category);
		new Category(category2);
		ArrayList<Item> itemResult = new ArrayList<Item>();
		ArrayList<Item> emptyArray = new ArrayList<Item>();

		Item i = new Item();
		i.setTitle(title);
		i.setCategory(Category.getCategory(category));

		itemResult.add(i);

		assertEquals(u.searchItem(title, category), itemResult);
		assertEquals(u.searchItem("ferrari", category), emptyArray);
		assertEquals(u.searchItem(title, category2), emptyArray);
		assertEquals(u.searchItem("ferrari", category2), emptyArray);

	}

	@TestFactory
	public Collection<DynamicTest> searchAuctionAcceptanceTest() {

		ArrayList<String> title = new ArrayList<String>(Arrays.asList("", "9999", "t-#\\",
				"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa", "title", "title", "title"));
		ArrayList<String> category = new ArrayList<String>(
				Arrays.asList("camera", "camera", "camera", "camera", "bike", "camera", "camera"));
		ArrayList<Integer> price = new ArrayList<Integer>(Arrays.asList(20, 20, 20, 20, 20, 0, -1));
		ArrayList<String> testNames = new ArrayList<String>(Arrays.asList("title empty", "numerical title",
				"symbols present in the title", "title longer than 30 characters", "cateogry not correct",
				"price eqaul to zero", "price is a negative number"));
		ArrayList<Class<? extends Exception>> exceptions = new ArrayList<Class<? extends Exception>>(
				Arrays.asList(TitleInvalidException.class, TitleInvalidException.class, TitleInvalidException.class,
						TitleInvalidException.class, CategoryInvalidException.class, PriceInvalidException.class,
						PriceInvalidException.class));

		Collection<DynamicTest> dynamicTests = new ArrayList<DynamicTest>();

		new Category("camera");
		User u = User.getInstance();

		for (int i = 0; i < title.size(); i++) {

			Item item = Item.getInstance();
			item.setTitle(title.get(i));

			final int j = i;

			Executable exec = () -> assertThrows(exceptions.get(j),
					() -> u.searchAuction(price.get(j), title.get(j), category.get(j)));
			DynamicTest dTest = DynamicTest.dynamicTest(testNames.get(i), exec);

			dynamicTests.add(dTest);

		}

		return dynamicTests;

	}

	@Test
	public void searchAuctionTest()
			throws CategoryInvalidException, TitleInvalidException, PriceInvalidException, ParseException {

		String format = "dd/MM/yyyy hh:mm:ss";
		SimpleDateFormat sdf = new SimpleDateFormat(format);

		String title = "Red bike", category = "Sport and Outdoor", category2 = "Cars and Motorbikes";
		Date start = null, end = null;

		start = sdf.parse("22/06/2020 11:20:12");
		end = sdf.parse("22/07/2020 11:20:12");

		double price = 10;
		int bid = 5;
		User u = User.getInstance();
		new Category(category);
		new Category(category2);
		ArrayList<Auction> auctionResult = new ArrayList<Auction>();
		ArrayList<Auction> emptyArray = new ArrayList<Auction>();

		Item i = new Item();
		i.setTitle(title);
		i.setCategory(Category.getCategory(category));

		Auction a = new Auction(i, start, end, price, bid);

		auctionResult.add(a);

		assertEquals(u.searchAuction(price, title, category), auctionResult);
		assertEquals(u.searchAuction(price + 1, title, category), auctionResult);
		assertEquals(u.searchAuction(price, "ferrari", category), emptyArray);
		assertEquals(u.searchAuction(price, title, category2), emptyArray);
		assertEquals(u.searchAuction(price - 1, "ferrari", category2), emptyArray);

	}

	@TestFactory
	public Collection<DynamicTest> searchItemAcceptanceTest() {

		ArrayList<String> title = new ArrayList<String>(
				Arrays.asList("", "9999", "t-#\\", "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa", "title"));
		ArrayList<String> category = new ArrayList<String>(
				Arrays.asList("camera", "camera", "camera", "camera", "bike"));
		ArrayList<String> testNames = new ArrayList<String>(Arrays.asList("title empty", "numerical title",
				"symbols present in the title", "title longer than 30 characters", "cateogry not correct"));
		ArrayList<Class<? extends Exception>> exceptions = new ArrayList<Class<? extends Exception>>(
				Arrays.asList(TitleInvalidException.class, TitleInvalidException.class, TitleInvalidException.class,
						TitleInvalidException.class, CategoryInvalidException.class));

		Collection<DynamicTest> dynamicTests = new ArrayList<DynamicTest>();

		new Category("camera");
		User u = User.getInstance();

		for (int i = 0; i < title.size(); i++) {

			Item item = Item.getInstance();
			item.setTitle(title.get(i));

			final int j = i;

			Executable exec = () -> assertThrows(exceptions.get(j), () -> u.searchItem(title.get(j), category.get(j)));
			DynamicTest dTest = DynamicTest.dynamicTest(testNames.get(i), exec);

			dynamicTests.add(dTest);

		}

		return dynamicTests;

	}

	@ParameterizedTest
	@ValueSource(ints = { 0, -1, -20 })
	public void searchItemError(int userId) {

		String title = "Red Bike";
		String category = "Sport and Outdoor";

		new Category(category);

		Item i = new Item();
		i.setTitle(title);
		i.setCategory(Category.getCategory(category));

		User u = User.getInstance();
		assertThrows(PriceInvalidException.class, () -> u.searchAuction(userId, title, category));

	}

	@Test
	public void addItemTest() {

		int id1 = 1;

		User u = User.getInstance();
		Item i = Item.getInstance();
		i.setId(id1);

		assertTrue(u.addItemToList(id1));
		assertFalse(u.addItemToList(id1));

	}

	@Test
	public void getUserByIdTest() {

		int id1 = 1;

		User u = User.getInstance();
		u.setId(id1);

		assertEquals(u, User.getUserById(id1));
		assertNull(User.getUserById(2));

	}

}
